"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Modules
var md5_1 = require("md5");
var base64 = require("base-64");
var classNames = require("classnames");
// Wordings
var wordings_1 = require("@amalto/wordings");
// Constants
var Config_1 = require("./constants/Config");
var Data_1 = require("./constants/Data");
exports.EMAIL_REGEX = /^\S+@\S+\.\S+$/;
exports.COLOR_CODE_REGEX = /^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/;
exports.SCOPE_KEYWORD_REGEX = /^[a-zA-Z0-9-_~@$£|€¥§&]+$/;
exports.MAP_PROPERTY_KEY_REGEX = /^[a-zA-Z0-9-_]+$/;
exports.XML_TAG_REGEX = /^[a-zA-Z_:][a-zA-Z0-9_:\-\.]*$/;
exports.HTTPS_URL_REGEX = /https:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&\/=]*)/;
function getWordingsInUserLanguage(locale) {
    var res = Object.keys(wordings_1.MULTILANGUAGE_WORDINGS).reduce(function (dic, key) {
        dic[key] = wordings_1.MULTILANGUAGE_WORDINGS[key][locale];
        return dic;
    }, {});
    return res;
}
exports.getWordingsInUserLanguage = getWordingsInUserLanguage;
function compileWordings(wordings, locale) {
    locale = locale || "en-US";
    var res = Object.keys(wordings).reduce(function (dic, key) {
        dic[key] = wordings[key][locale];
        return dic;
    }, {});
    return res;
}
exports.compileWordings = compileWordings;
/** Return gravater linked to use email. */
function getGravatarUrl(email) {
    var baseUrl = 'https://secure.gravatar.com/avatar/';
    var queryParams = '?s=200&d=mm';
    return baseUrl + md5_1.default(email.trim().toLowerCase()) + queryParams;
}
exports.getGravatarUrl = getGravatarUrl;
function isValidPassword(password) {
    var minMaxLength = /^[\s\S]{8,32}$/, upper = /[A-Z]/, lower = /[a-z]/, number = /[0-9]/, count = 0;
    if (minMaxLength.test(password)) {
        if (upper.test(password)) {
            count++;
        }
        if (lower.test(password)) {
            count++;
        }
        if (number.test(password)) {
            count++;
        }
    }
    return count >= 2;
}
exports.isValidPassword = isValidPassword;
//at least it has an "@" and a "." in it...
function isValidEmail(email) {
    return exports.EMAIL_REGEX.test(email);
}
exports.isValidEmail = isValidEmail;
//check for an hex color code like #fff or #ffffff
function isValidColorCode(color) {
    return exports.COLOR_CODE_REGEX.test(color);
}
exports.isValidColorCode = isValidColorCode;
function isNotEmpty(value) {
    return !!value && value.trim().length > 0;
}
exports.isNotEmpty = isNotEmpty;
//check for a valid scope keyword value (see "Scopes and Permissions" page on confluence)
function isValidScopeKeyword(value) {
    return exports.SCOPE_KEYWORD_REGEX.test(value);
}
exports.isValidScopeKeyword = isValidScopeKeyword;
//check for a valid object key char
function isValidKeyChar(value) {
    return exports.MAP_PROPERTY_KEY_REGEX.test(value);
}
exports.isValidKeyChar = isValidKeyChar;
//check for a valid XML Tag
function isValidXMLTag(value) {
    return exports.XML_TAG_REGEX.test(value);
}
exports.isValidXMLTag = isValidXMLTag;
// Check for a valid https url
function isValidHttpsUrl(value) {
    return exports.HTTPS_URL_REGEX.test(value);
}
exports.isValidHttpsUrl = isValidHttpsUrl;
function escapeXml(xml) {
    return xml.replace(/[<>&'"]/g, function (c) {
        switch (c) {
            case '<':
                return '&lt';
            case '>':
                return '&gt';
            case '&':
                return '&amp';
            case '\'':
                return '&apos';
            case '"':
                return '&quot';
        }
    });
}
exports.escapeXml = escapeXml;
function utf8JSON_to_b64URI(json) {
    return encodeURIComponent(base64.encode(decodeURIComponent(encodeURIComponent(JSON.stringify(json)))));
}
exports.utf8JSON_to_b64URI = utf8JSON_to_b64URI;
function URIb64_to_utf8JSON(str) {
    try {
        return JSON.parse(decodeURIComponent(encodeURIComponent(base64.decode(decodeURIComponent(str)))));
    }
    catch (error) {
        console.log('JSON parsing error: ', error);
        return {};
    }
}
exports.URIb64_to_utf8JSON = URIb64_to_utf8JSON;
function arrayMin(arr) {
    return arr.reduce(function (prev, next) {
        return (prev < next ? prev : next);
    });
}
exports.arrayMin = arrayMin;
function arrayMax(arr) {
    return arr.reduce(function (prev, next) {
        return (prev > next ? prev : next);
    });
}
exports.arrayMax = arrayMax;
function formatFileSize(size) {
    if (!size) {
        return '0 B';
    }
    var k = 1000;
    var sizes = ['B', 'KB', 'MB', 'GB', 'TB'];
    var i = Math.floor(Math.log(size) / Math.log(k));
    var unit = i < 5 ? sizes[i] : sizes[4];
    return (size / Math.pow(k, i)).toFixed(1) + ' ' + unit;
}
exports.formatFileSize = formatFileSize;
/**
 * Returns a key-value object from a query string like ?test=value&other=something
 */
function getQueryParams(searchString) {
    if (!searchString) {
        return {};
    }
    var queryStringParams = searchString.substr(1).split('&');
    var queryParams = queryStringParams.reduce(function (acc, queryStrParam) {
        var keyValue = queryStrParam.split('=');
        if (keyValue.length === 2) {
            acc[keyValue[0]] = decodeURIComponent(keyValue[1]);
        }
        return acc;
    }, {});
    return queryParams;
}
exports.getQueryParams = getQueryParams;
/**
 * Adds a query parameter to a URI (string)
 * returns an HTMLAnchorElement
 */
function addQueryParam(uri, key, value) {
    var link = document.createElement('a');
    link.href = uri;
    if (link.search) {
        link.search += ('&' + encodeURIComponent(key) + '=' + encodeURIComponent(value));
    }
    else {
        link.search = ('?' + encodeURIComponent(key) + '=' + encodeURIComponent(value));
    }
    return link;
}
exports.addQueryParam = addQueryParam;
//return the same object passed as param but with ASC ordered keys
function orderAsc(object) {
    var result = {};
    Object.keys(object).sort().forEach(function (key) {
        result[key] = object[key];
    });
    return result;
}
exports.orderAsc = orderAsc;
//return the same object passed as param but with DESC ordered keys
function orderDesc(object) {
    var result = {};
    Object.keys(object).sort().reverse().forEach(function (key) {
        result[key] = object[key];
    });
    return result;
}
exports.orderDesc = orderDesc;
function saveDataAsJSONFile(data, fileName) {
    var json = JSON.stringify(data, null, '\t');
    var blob = new Blob([json], {
        type: 'application/json'
    });
    triggerDataDownload(blob, fileName + '_' + (new Date().toISOString().substr(0, 19)) + '.json');
}
exports.saveDataAsJSONFile = saveDataAsJSONFile;
function downloadDataFile(base64DataString, contentType, fileName) {
    var b64String = 'data:' + contentType + ';base64,' + base64DataString;
    triggerDataDownload(b64String, fileName, true);
}
exports.downloadDataFile = downloadDataFile;
function triggerDataDownload(data, fileName, dataUrl) {
    var url = dataUrl ? data : URL.createObjectURL(data);
    //create a download link
    var link = document.createElement('a');
    link.href = url;
    link['download'] = fileName;
    //trigger the download
    link.click();
    URL.revokeObjectURL(url);
}
function hasRequiredResource(appEndpoints, appInstanceName, featureId) {
    //search for an existing endpoint for this feature on the current instance
    if (appEndpoints && appEndpoints[appInstanceName]) {
        if (appEndpoints[appInstanceName][featureId]) {
            return true;
        }
    }
    return false;
}
exports.hasRequiredResource = hasRequiredResource;
function replaceTemplateViewName(templatedHtml, viewName) {
    return templatedHtml.replace(/{{VIEWNAME}}/g, viewName);
}
exports.replaceTemplateViewName = replaceTemplateViewName;
//SPECIAL TRANSFORMERS/ADPATERS HELPERS
function replaceTemplateFlags(templatedHtml, locale) {
    var flagsRegex = /{{FLAGS\=(.*)}}/g;
    var matchRes = flagsRegex.exec(templatedHtml);
    if (matchRes && matchRes.length === 2) {
        var flags = matchRes[1].split(',');
        var flagsHtml = '';
        for (var flagName in Data_1.flagsDef) {
            var flagDef = Data_1.flagsDef[flagName];
            if (flagName !== 'eunread') {
                if ((flagDef.inversed && flags.indexOf(flagName) === -1) || (!flagDef.inversed && flags.indexOf(flagName) !== -1)) {
                    flagsHtml += '<span class="fa-fw right-spaced text-xlarge ' + flagDef.iconColor + ' ' + flagDef.iconShape + '" title="' + flagDef.flagLabel[locale] + '"></span>';
                }
            }
        }
        return templatedHtml.replace(flagsRegex, flagsHtml);
    }
    else {
        return templatedHtml;
    }
}
exports.replaceTemplateFlags = replaceTemplateFlags;
function getStyleDef(styleConf) {
    var styles = styleConf.split(',');
    var res = {
        icon: null,
        btn: null,
        color: null
    };
    styles.forEach(function (style) {
        if (style.indexOf('icon:') !== -1) {
            res.icon = style.replace('icon:', '');
        }
        if (style.indexOf('btn:') !== -1) {
            res.btn = style.replace('btn:', '');
        }
        if (style.indexOf('color:') !== -1) {
            res.color = style.replace('color:', '');
        }
    });
    return res;
}
exports.getStyleDef = getStyleDef;
function getAcceptLanguageHeader(locale) {
    var notSelected = Config_1.AVAILABLE_LANGUAGES.filter(function (language, idx) {
        //limit to 3 languages max
        return language.locale !== locale && idx < 3;
    }).map(function (language, idx) {
        //decreasing priority for each not selected language
        return language.locale.substr(0, 2).toLowerCase() + ';q=' + (0.8 - (idx / 10)).toString();
    });
    return locale.substr(0, 2).toLowerCase() + ',' + notSelected.join(',');
}
exports.getAcceptLanguageHeader = getAcceptLanguageHeader;
function getI18nLabel(locale, labelMap, noRegion, upper) {
    var res = null;
    var language = locale;
    if (noRegion) {
        language = language.substr(0, 2);
    }
    if (upper) {
        language = language.toUpperCase();
    }
    if (labelMap) {
        if (labelMap[language]) {
            res = labelMap[language];
        }
        if (!res && language !== 'EN') {
            res = labelMap['EN'];
        }
        if (!res && language !== 'en') {
            res = labelMap['en'];
        }
        if (!res && language !== 'en-US') {
            res = labelMap['en-US'];
        }
    }
    return res;
}
exports.getI18nLabel = getI18nLabel;
function getJSTreeData(orgTreeData, openedNodes) {
    return orgTreeData ? {
        id: orgTreeData.id,
        text: orgTreeData.id === '0' ? '' : orgTreeData.elementName,
        data: {
            description: orgTreeData.description,
            propertiesMap: orgTreeData.propertiesMap,
            parentId: orgTreeData.parentId,
            childNames: orgTreeData.children ? orgTreeData.children.map(function (child) { return child.elementName; }) : []
        },
        children: orgTreeData.children ? orgTreeData.children.map(function (child) {
            return getJSTreeData(child, openedNodes);
        }) : null,
        icon: classNames('fas fa-fw', {
            'fa-th-large font-color-lighter': orgTreeData.id !== '0',
            'fa-terminal black-color': orgTreeData.id === '0',
        }),
        state: {
            opened: openedNodes ? openedNodes.indexOf(orgTreeData.id) !== -1 : false,
            //fake root node created when the user has multiple positions at the same level should be disabled
            disabled: orgTreeData.id === '-1'
        }
    } : null;
}
exports.getJSTreeData = getJSTreeData;
function loadTooltips(element) {
    if (!Modernizr.touchevents) {
        var $elementTooltips_1 = $(element).find('[data-toggle="tooltip"]');
        $elementTooltips_1.tooltip({
            container: 'body',
            placement: 'auto top'
        });
        $elementTooltips_1.on('click', function () {
            $elementTooltips_1.tooltip('hide');
        });
    }
}
exports.loadTooltips = loadTooltips;
function unloadTooltips(element) {
    if (!Modernizr.touchevents) {
        var $elementTooltips = $(element).find('[data-toggle="tooltip"]');
        $elementTooltips.tooltip('hide');
        $elementTooltips.tooltip('destroy');
        $('div.tooltip').remove();
    }
}
exports.unloadTooltips = unloadTooltips;
function groupByProperty(list, propertyName) {
    return list.reduce(function (grouped, item) {
        var key = item[propertyName];
        grouped[key] = grouped[key] || [];
        grouped[key].push(item);
        return grouped;
    }, {});
}
exports.groupByProperty = groupByProperty;
/**
 * Add an element if it isn't already in the array
 * @param array Array to process
 * @param value Value to add
 */
function addValToArrayNoDup(array, value) {
    if (array.indexOf(value) === -1) {
        return array.concat(value);
    }
    return array.slice();
}
exports.addValToArrayNoDup = addValToArrayNoDup;
/**
 * Remove all element from array that match the value parameter
 * @param array Array to process
 * @param value Value to exclude
 */
function removeValFromArrayNoDup(array, value) {
    return array.filter(function (element) { return element !== value; });
}
exports.removeValFromArrayNoDup = removeValFromArrayNoDup;
/**
 * Get nested value from object and pointed by a doted path
 * @param obj Object to search in
 * @param keyPath Path to search to
 */
function getNestedValue(obj, keyPath) {
    if (!obj) {
        return undefined;
    }
    var props = keyPath.split('.');
    if (props.length === 1) {
        return obj[props[0]];
    }
    else {
        return getNestedValue(obj[props[0]], props.slice(1).join('.'));
    }
}
exports.getNestedValue = getNestedValue;
/**
 * Filter collection by given  properties and matching string
 * @param collection Array to filter
 * @param properties Properties keys to sort the array by
 * @param searchString If given, only return value that contain this value
 */
function filterCollection(collection, properties, searchString) {
    var searchElements = searchString.toLowerCase().split(' ');
    var filteredCollection = searchString ? collection.filter(function (item) {
        var data = properties.map(function (property) {
            if (property.split('.').length > 1) {
                return getNestedValue(item, property);
            }
            else {
                return item[property];
            }
        }).join().toLowerCase();
        var matches = searchElements.map(function (search) { return data.indexOf(search) !== -1; });
        return matches.filter(function (matched) { return !matched; }).length === 0;
    }) : collection;
    return filteredCollection;
}
exports.filterCollection = filterCollection;
function base64Decode(encodedData) {
    return base64.decode(encodedData);
}
exports.base64Decode = base64Decode;
/**
 * Copy object data into another object
 * @param data Object to copy and extend
 * @param extensions Object to inject into data
 */
function deepCopy(data, extensions) {
    return !extensions ? JSON.parse(JSON.stringify(data || {})) : $.extend({}, data, extensions);
}
exports.deepCopy = deepCopy;
/**
 * @description Handle dupplicate name from array
 * @param { string } name Name to check
 * @param { string[] } container String to compared to
 */
function handleDuplicateNameFromArray(name, container) {
    var firstTime = true;
    var res = name;
    var idx = 1;
    while (container.some(function (c) { return res === c; })) {
        if (firstTime) {
            res = res.concat("_" + idx);
        }
        else {
            res = res.replace(/_[\d]+$/, "_" + idx);
        }
        firstTime = false;
        idx++;
    }
    return res;
}
exports.handleDuplicateNameFromArray = handleDuplicateNameFromArray;
/**
 * Return string date from timestamp and locale.
 * @param timestamp timestamp in milliseconde
 * @param locale locale to be used
 * @param options options of toLocaleString method
 */
function dateByLocalToString(locale, date, options) {
    return new Date(date).toLocaleString(locale, deepCopy({
        month: 'numeric',
        day: 'numeric',
        year: 'numeric',
        hour: 'numeric',
        minute: 'numeric'
    }, options));
}
exports.dateByLocalToString = dateByLocalToString;
//# sourceMappingURL=index.js.map