"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Constants
var ASTERISK = '*';
/**
 * User got permission.
 * e.g: reports=view
 */
function hasPermission(webStorage, permission, global) {
    return hasAnyPermission(webStorage, [permission], global);
}
exports.hasPermission = hasPermission;
/**
 * User got all permissions.
 * e.g: ['reports=view', 'reports=read']
 */
function hasAnyPermission(webStorage, permissions, global) {
    var appInstance = webStorage.selectedAppInstance && webStorage.selectedAppInstance.name;
    if (!permissions || !permissions.length || !appInstance) {
        return false;
    }
    for (var i = 0; i < permissions.length; i++) {
        if (typeof permissions[i] === 'object') {
            //if the permission is required globally, the check will be only on the * instance
            var appInstanceValues = getScopeValues(webStorage, global ? ASTERISK : appInstance, permissions[i].feature, permissions[i].action);
            if (appInstanceValues) {
                if ($.isEmptyObject(appInstanceValues)) {
                    return true;
                }
                if (appInstanceValues.length) {
                    if (appInstanceValues.indexOf(permissions[i].requiredValue) !== -1) {
                        return true;
                    }
                }
            }
            //check values on * instance            
            if (!global) {
                var asteriskValues = getScopeValues(webStorage, ASTERISK, permissions[i].feature, permissions[i].action);
                if (asteriskValues) {
                    if ($.isEmptyObject(asteriskValues)) {
                        return true;
                    }
                    if (asteriskValues.length) {
                        if (asteriskValues.indexOf(permissions[i].requiredValue) !== -1) {
                            return true;
                        }
                    }
                }
            }
        }
        else if (typeof permissions[i] === 'string') {
            var parsedScopeString = permissions[i].split('=');
            if (parsedScopeString.length === 2) {
                if (hasPermissionFor(webStorage, global ? ASTERISK : appInstance, parsedScopeString[0], parsedScopeString[1])) {
                    return true;
                }
            }
            else {
                console.log('Error - bad scope string value:', permissions[i]);
            }
        }
    }
    return false;
}
exports.hasAnyPermission = hasAnyPermission;
/** @private */
function hasPermissionFor(webStorage, appInstance, feature, action) {
    return (hasAccessToAction(webStorage, appInstance, feature, action) ||
        hasAccessToAction(webStorage, appInstance, feature, ASTERISK) ||
        hasAccessToAction(webStorage, appInstance, ASTERISK, ASTERISK) ||
        hasAccessToAction(webStorage, ASTERISK, feature, action) ||
        hasAccessToAction(webStorage, ASTERISK, feature, ASTERISK) ||
        hasAccessToAction(webStorage, ASTERISK, ASTERISK, ASTERISK));
}
/**
 * Get item values from permission.
 * e.g: reports=read('Test'), will return ['Test'].
 */
function getScopeValues(webStorage, appInstance, feature, action) {
    var scopesTree = webStorage.scopesTree;
    return scopesTree && scopesTree[appInstance] && scopesTree[appInstance][feature] && scopesTree[appInstance][feature][action];
}
exports.getScopeValues = getScopeValues;
/**
 * User has access to feature id.
 * e.g: reports=read, here 'reports' is the feature id.
 */
function hasAccessToFeature(webStorage, appInstance, feature, global) {
    var scopesTree = webStorage.scopesTree;
    if (!scopesTree || !appInstance || !feature) {
        return false;
    }
    var featuresMap = scopesTree[global ? ASTERISK : appInstance];
    for (var _feature in featuresMap) {
        if (_feature === feature || _feature === ASTERISK) {
            return true;
        }
    }
    //if we don't find an access to the feature on the specific instance, we can check on the * one
    if (!global) {
        return hasAccessToFeature(webStorage, appInstance, feature, true);
    }
    return false;
}
exports.hasAccessToFeature = hasAccessToFeature;
/**
 * User has access to feature action.
 * e.g: reports=read, here 'read' is the action.
 */
function hasAccessToAction(webStorage, appInstance, feature, action) {
    if (!appInstance || !feature || !action) {
        return false;
    }
    if (hasAccessToFeature(webStorage, appInstance, feature)) {
        return getScopeValues(webStorage, appInstance, feature, action) !== undefined;
    }
    else {
        return false;
    }
}
/**
 * Action has filter on it.
 */
function hasFilterOn(webStorage, feature, viewFilter, filterKey) {
    var appInstance = webStorage.selectedAppInstance && webStorage.selectedAppInstance.name;
    //filter by key value are made via the keyword 'allow'
    var appInstanceValues = appInstance ? getScopeValues(webStorage, appInstance, feature, 'allow') : undefined;
    var asteriskValues = getScopeValues(webStorage, ASTERISK, feature, 'allow');
    if (appInstanceValues && appInstanceValues[viewFilter]) {
        if (appInstanceValues[viewFilter].filter(function (scopeFilter) { return scopeFilter.searchable === filterKey; }).length > 0) {
            return true;
        }
    }
    if (asteriskValues && asteriskValues[viewFilter]) {
        if (asteriskValues[viewFilter].filter(function (scopeFilter) { return scopeFilter.searchable === filterKey; }).length > 0) {
            return true;
        }
    }
    return false;
}
exports.hasFilterOn = hasFilterOn;
/**
 * True if only one requiredActions is contain within the authorizedActions.
 */
function canPerformAnyAction(requiredActions, authorizedActions) {
    if (!requiredActions || !requiredActions.length || !authorizedActions) {
        return false;
    }
    //if AT LEAST ONE requiredActions is in the authorized ones it is OK
    for (var i = 0; i < requiredActions.length; i++) {
        if (authorizedActions.indexOf(requiredActions[i]) !== -1) {
            return true;
        }
    }
    return false;
}
exports.canPerformAnyAction = canPerformAnyAction;
//# sourceMappingURL=index.js.map